@@ -1,4 +1,5 @@
 import warnings
+from textwrap import dedent
 from . import expected
 from gffutils import example_filename, create, parser, feature
 import gffutils
@@ -94,20 +95,23 @@ def test_update():
     # IDs.
     db = create.create_db(example_filename('FBgn0031208.gff'), ':memory:',
                           verbose=False, force=True, keep_order=True)
-    for gene in db.features_of_type('gene'):
-        for child in list(db.children(gene)):
-            # important: the FBgn0031208.gff file was designed to have some
-            # funky features: there are two exons without ID attributes.  These
-            # are assigned to ids "exon_1" and "exon_2".  Upon update, with
-            # still no ID, we then have two new features "exon_3" and "exon_4".
-            # To prevent this issue, we ensure that the ID attribute exists...
-            child.attributes['gene_id'] = [gene.id]
-            if 'ID' not in child.attributes:
-                child.attributes['ID'] = [child.id]
-            db.update([child], merge_strategy='replace')
+    def gen():
+        for gene in db.features_of_type('gene'):
+            for child in list(db.children(gene)):
+                # important: the FBgn0031208.gff file was designed to have some
+                # funky features: there are two exons without ID attributes.  These
+                # are assigned to ids "exon_1" and "exon_2".  Upon update, with
+                # still no ID, we then have two new features "exon_3" and "exon_4".
+                # To prevent this issue, we ensure that the ID attribute exists...
+                child.attributes['gene_id'] = [gene.id]
+                if 'ID' not in child.attributes:
+                    child.attributes['ID'] = [child.id]
+                yield child
+
+    db.update(gen(), merge_strategy='replace')
 
     print("\n\nafter\n\n")
-    for child in db.children(gene):
+    for child in db.children('FBgn0031208'):
         print(child.id)
         assert child.attributes['gene_id'] == ['FBgn0031208'], (child, child.attributes)
 
@@ -198,7 +202,7 @@ class BaseDB(object):
         self.dialect = self.db.dialect
 
     def table_test(self):
-        expected_tables = ['features', 'relations', 'meta', 'directives', 'autoincrements', 'duplicates']
+        expected_tables = ['features', 'relations', 'meta', 'directives', 'autoincrements', 'duplicates', 'sqlite_stat1']
         self.c.execute('select name from sqlite_master where type="table"')
         observed_tables = [i[0] for i in self.c.execute('select name from sqlite_master where type="table"')]
         assert set(expected_tables) == set(observed_tables), observed_tables
@@ -1027,6 +1031,149 @@ def test_infer_gene_extent():
         assert len(w) == 1
 
 
+# From #79
+def test_issue_79():
+    gtf = gffutils.example_filename('keep-order-test.gtf')
+    db = gffutils.create_db(gtf, 'tmp.db',
+                       disable_infer_genes=False,
+                       disable_infer_transcripts=False,
+                       id_spec={"gene": "gene_id", "transcript": "transcript_id"},
+                       merge_strategy="create_unique",
+                       keep_order=True,
+                            force=True)
+
+    exp = open(gtf).read()
+    obs = '\n'.join([str(i) for i in db.all_features()])
+    exp_1 = exp.splitlines(True)[0].strip()
+    obs_1 = obs.splitlines(True)[0].strip()
+    print('EXP')
+    print(exp_1)
+    print('OBS')
+    print(obs_1)
+    print('DIFF')
+    print(''.join(difflib.ndiff([exp_1], [obs_1])))
+    assert obs_1 == exp_1
+
+def test_for_analyze():
+    db = gffutils.create_db(
+            gffutils.example_filename('FBgn0031208.gtf'),
+            'deleteme',
+            force=True
+    )
+    assert db._analyzed()
+    db.execute('DROP TABLE sqlite_stat1')
+    assert not db._analyzed()
+
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter("always")
+        db2 = gffutils.FeatureDB('deleteme')
+        assert len(w) == 1
+        assert "analyze" in str(w[-1].message)
+    db.analyze()
+    assert db._analyzed()
+    os.unlink('deleteme')
+
+
+def test_issue_82():
+    # key-val separator is inside an unquoted attribute value
+    x = (
+        'Spenn-ch12\tsgn_markers\tmatch\t2621812\t2622049\t.\t+\t.\t'
+        'Alias=SGN-M1347;ID=T0028;Note=marker name(s): T0028 SGN-M1347 |identity=99.58|escore=2e-126'
+    )
+    y = feature.feature_from_line(x)
+    assert y.attributes['Note'] == ['marker name(s): T0028 SGN-M1347 |identity=99.58|escore=2e-126']
+
+    gffutils.create_db(gffutils.example_filename('keyval_sep_in_attrs.gff'), ':memory:')
+
+def test_sequence():
+    fasta = gffutils.example_filename('dm6-chr2L.fa')
+    f = feature.feature_from_line(
+        'chr2L	FlyBase	gene	154	170	.	+	.	ID=one;')
+    seq = f.sequence(fasta)
+    assert seq == 'aCGAGATGATAATATAT'
+    assert len(seq) == len(f)
+    f.strand = '-'
+    seq = f.sequence(fasta)
+    assert seq == 'ATATATTATCATCTCGt'
+    assert len(seq) == len(f)
+
+def test_issue_85():
+    # when start or stop was empty, #85 would fail Should now work with
+    # blank fields
+    f = feature.feature_from_line('\t'.join([''] * 9))
+
+    # or with "." placeholders
+    f = feature.feature_from_line('\t'.join(['.'] * 9))
+
+
+def test_unquoting():
+    # incoming is encoded
+    s = (
+        'chr1\tAUGUSTUS\tgene\t6950084\t6951407\t0.26\t-\t.\t'
+        'ID=INIL01g00009;GeneSymbol=Ndufaf6;Note=NADH dehydrogenase '
+        '(ubiquinone) complex I%2C assembly factor 6;GO_Terms=GO:0005743|'
+        'GO:0016740|GO:0009058|GO:0032981;PFam=PF00494'
+    )
+    f = feature.feature_from_line(s, keep_order=True)
+
+    # string representation should be identical
+    assert str(f) == s
+
+    # accessing attribute should be decoded
+    n = f['Note']
+    assert n == ['NADH dehydrogenase (ubiquinone) complex I, assembly factor 6']
+
+
+def test_unreasonable_unquoting():
+    s = (
+        'chr1\t.\t.\t1\t2\t0.26\t-\t.\t'
+        'newline=%0A;'
+        'percent=%25;'
+        'null=%00;'
+        'comma=%2C;'
+
+        # The first parent is "A," (A with a comma), the second is "B%"
+        'Parent=A%2C,B%25,C;'
+    )
+    f = feature.feature_from_line(s, keep_order=True)
+    assert f.attributes['newline'][0] == '\n'
+    assert f.attributes['percent'][0] == '%'
+    assert f.attributes['null'][0] == '\x00'
+    assert f.attributes['comma'][0] == ','
+
+    # Commas indicate 
+    assert f.attributes['Parent'] == ['A,', 'B%', 'C']
+    assert str(f) == s
+
+
+def test_unquoting_iter():
+    s = 'chr1\t.\tgene\t1\t2\t.\t-\t.\tID=%2C;'
+    tmp = tempfile.NamedTemporaryFile(delete=False).name
+    with open(tmp, 'w') as fout:
+        fout.write(s + '\n')
+    assert list(gffutils.iterators.DataIterator(tmp))[0]['ID'][0] == ','
+
+def test_db_unquoting():
+    s = dedent(
+        '''
+        chr1\t.\tgene\t1\t2\t.\t-\t.\tID=a;Note=%2C;
+        chr1\t.\tgene\t1\t2\t.\t-\t.\tID=b;Note=%2C;
+        chr1\t.\tgene\t1\t2\t.\t-\t.\tID=c;Note=%2C;
+        chr1\t.\tgene\t1\t2\t.\t-\t.\tID=d;Note=%2C;
+        chr1\t.\tgene\t1\t2\t.\t-\t.\tID=e;Note=%2C;
+        chr1\t.\tgene\t1\t2\t.\t-\t.\tID=f;Note=%2C;
+        ''')
+    tmp = tempfile.NamedTemporaryFile(delete=False).name
+    with open(tmp, 'w') as fout:
+        fout.write(s + '\n')
+    db = gffutils.create_db(tmp, ':memory:', checklines=1)
+    assert db['a']['Note'] == [',']
+    assert db['b']['Note'] == [',']
+    assert db['c']['Note'] == [',']
+    assert db['d']['Note'] == [',']
+    assert db['e']['Note'] == [',']
+    assert db['f']['Note'] == [',']
+
 if __name__ == "__main__":
     # this test case fails
     #test_attributes_modify()
